.\" vi:sw=1:tw=78:ts=4
.AM
.\" program start
.de Ps
.DS
.vs -2p
.ss 23
..
.\" program end
.de Pe
.vs +2p
.ss 12
.DE
..
.\" paragraph with less vertical spacing in header
.de Pp
.PP
.sp -3p
..
.\" dito, with .LP
.de Lp
.LP
.sp -3p
..
.\" new section header (idented)
.de Sh
.SH
.B
\h'2n'\\$*
.R
.sp 0.5v
..
.\" description header
.de Dh
.SH
\s-1\h'2n'\\$*\s0
..
.\" description start
.de Ds
.IP
.ll -6n
\(em
'in +\w'\(em 'u+1u
..
.\" description end
.de De
.ll
.in
.br
..
.\" quotes
.ds lq ``
.ds rq ''
.\" mathematical lower sign
.ds lo \h'2p'\v'-1p'<\v'+1p'\h'2p'
.\" foot note daggers; first for text, second for note
.ie n .ds f1 \(**.
.el   .ds f1 \v'-4p'\s-4\(dg\s0\v'4p'.
.ie n .ds f2 \(**
.el   .ds f2 \v'-2p'\(dg\v'2p'
.de DT
.if t .ta .5i 1.25i 2.5i 3.75i
.if n .ta .7i 1.75i 3.8i
..
.TL
\s+5Fixed-point Extension for the CGC Domain\s0
.br
.AU
.sp 0.3v
\s+3Ju\*:rgen Weiss, Universita\*:t Stuttgart, Germany\s0
.sp 0.6v
\s+3version: 11/19/94\s0
.AB no
This draft describes an extension to the \fBPTOLEMY\fP simulation system that
allows the representation of portholes and states as fixed-point values
to simulate calculations with limited precision (that is limited word
length) and constant normalization factor.
.br
The extension does not depend on large and possibly unrevertable kernel
changes, so that it should fit neatly into the code generation framework of
the CGC domain.  Furthermore it was designed with the Fix class implementation
in mind, so that transforming stars from the SDF into the CGC domain should be
fairly easy.
.AE
.nr H1 1
.\"
.\"
.Sh I. Concepts
.PP
A \fIfix value\fP (that is the value of some fixed-point particle of a
porthole or the value of a state of type fix) is described in the generated
code by the triplet \*Q\fILength\fP,\fIIntBits\fP,\fIVariable\fP\*Q.  \fILength\fP
is the total word length used to represent the value as a bit pattern,
\fIIntBits\fP determines the size of the integer part in the bit pattern and
therefore the position of the binary point, and \fIVariable\fP is a pointer to
the bit pattern itself implemented as an instance of the predefined C type
\*Q\fIfix\fP\*U (an array of words).
.LP
Referencing a fix value through the codeblock macros \*Q\fI$ref(name)\fP\*U or
\*Q\fI$ref(name,offset)\fP\*U will automatically expand into a fix triplet so
that the programmer very seldomly needs to deal with the triplet by himself.
Instead he will use the predefined functions and macros of the CGC runtime
library that provide the means to handle the triplets in all fixed-point
operations, like assignments, code convertion, arithmetic operations or
comparisons.  Beyond that, it is unwise to generate code that depends on the
format of a fix triplet, since the triplet might be extended in future (for
example to provide the means to specify overflow handling or rounding modes).
For this purpose, the library provides implementation independent macros to
reference the individual triplet elements.
.PP
The \fILength\fP and \fIIntBits\fP items of the fix triplet, I will refer to
them as the \*Q\fIprecision\fP\*U of the fix, determine the representation of the
value as a bit pattern.  They must be set explicitly by the programmer using
the \*Q\fIsetPrecision\fP\*U methods of the involved porthole or state classes in the
same way as the SDF parameters of a porthole are set in the setup
routine\*(f1
.FS \*(f2
Precisions of states derived from other fix values must be set in the initCode
routine, as explained below.
.FE
A precision defined for an input port is taken over to the connected output
port (or vice versa), if a precision is not already defined for that port.  If
the precisions of the input and output ports differ, a star is inserted that
will handle the conversion between the two precisions appropriately.
.PP
Note that the precision pair, as it appears in the generated C code, will
consist of either integer constants or expressions of integer variables, as
the programmer has the ability to force the declaration of variables that
contain the actual precision of a fix value, if it is desirable to change the
precision at runtime.  This might be necessary to deal with overflow errors
that require augmenting the size of the integer part in the bit pattern and
eventually the augmentation of the total word length.
.PP
If a state or porthole is declared with the attribute \fIA_VARPREC\fP (as
opposed to \fIA_CONSTPREC\fP for values whose precision can be determined at
compile time), a structured variable of the C type \*Q\fIfix_prec\fP\*U will
be declared, containing the two fields \*Q\fIlen\fP\*U and \*Q\fIintb\fP\*U
that hold the \fILength\fP and \fIIntBits\fP field of the fix triplet,
respectively.  The new CGC macro \*Q\fI$precision\fP\*U expands into a
reference to this structure.  So, for example, a codeblock line of the form
\*Q\fI$precision(name).intb++;\fP\*Q can be used to increase the size of the
integer part in the representation of the state or porthole with the given
name.  Note that in addition to this, it will be necessary to shift the bit
pattern in general by using a function from the CGC runtime library, so that
the represented value remains unchanged.
.PP
In order to build an abstraction mechanism that frees the programmer from the
task of distinguishing between constant and variable precision representation,
a \fIclass Precision\fP is defined.  It contains the constant initial
precision and eventually the symbolic representation of the fix_prec variable.
.Lp
Precisions can be added or multiplied to compute the precision of the result
of adding or multipling two fix values of respective precisions (other
operators are implemented as well).  This is useful for computing the
precision of temporary variables needed to hold the results of intermediate
computations in nested arithmetic expressions, since the C language, unlike
C++ for example, does not provide the means to manage these temporary
variables automatically.
.PP
In the following example, a temporary variable (that is a C variable
represented by a non-settable state of type fix of a star) must be declared
with a precision large enough to hold the result of the addition of
\fIinput1\fP and \fIinput2\fP:
.KS
.Ps
        FIX_Add\|(\|$ref(temp),  $ref(input1),$ref(input2)\|);
        FIX_Mul\|(\|$ref(output), $ref(temp),\h'\w'input1'u-\w'temp'u'$ref(factor)\|);
.Pe
.Lp
[here \fIinput1\fP/\fI2\fP are ports of type fix, \fIfactor\fP may be a FixState \
and \fPtemp\fP is the described non-settable state].
The following line in the initCode routine of the star will set the precision of
the temp state appropriately:
.Ps
        temp.setPrecision\|(\|input1.precision() + input2.precision()\|);
.Pe
.Lp
.KE
The + operator of class Precision returns a constant precision large
enough to hold the result of the addition and, if the input ports are declared
with attribute \fIA_VARPREC\fP, it also contains a symbolic precision
literally composed of the references to the entries of the fix_prec structures
combined with C operators.  Therefore a \fI$ref\fP macro will expand correctly
into a symbolic expression including the variable (that is changing)
precisions of the operands.
.PP
Since the invocation order of the star's setup methods may not be
predetermined, a fix value that derives its precision from other values must
be set in the initCode rather than the setup method.  This is because a star's
setup method that references porthole precisions defined in the setup method
of a connected star would have to be invoked \fIafter\fP the connected star's
method.  In addition, some stars from the system library inspect the precision
of their input ports before they can decide whether to use variable precision
representation for their output ports or not.  Deriving the precision of a
value in the initCode method makes sure that this process is completed when
the precisions of the critical values are looked up.
.\"
.Sh II. The CGC runtime library
.PP
The CGC runtime library provides a set of functions and some few macros that
operate on fix triplets.  All functions return the maximum value representable
in a given precision in case of overflow (or the minimum value, in case of
underflow).  If the result has to be truncated to fit into the representation,
mathematical rounding is applied.
The library implementation ensures that functions and macros do not have side
effects and that source and destination operands may coincide.  This
simplifies the emulation of operations that change operands simultaneously
(like the \*Q+=\*U operator in C).
.PP
All library functions require one or more fix triplets passed as arguments
that result from the expansion of a \fI$ref\fP macro of a fix state or
porthole.  Eventually, the fix arguments can be weighted with the
\*Q\fIFIX_Weight\fP\*U macro by a power of 2.  The weighting is achieved by
modifying the \*Q\fIIntBits\fP\*U field of the fix triplet, so that no extra
bit shifting is required.  It is possible to weight source as well as
destination operands, though the definitition of the weighting factor differs
in the two cases (see the description of the macro below).
.PP
The library's definition resides in the file
\*Q$PTOLEMY/lib/cgc/CGCrtlib.c\*U, the associated header file is in
\*Q$PTOLEMY/lib/cgc/CGCrtlib.h\*U.  For both files symbolic links are created
automatically into the target directory where the generated code files live.
However, the library is not automatically linked into the generated
executable.  This has to be arranged explicitly by the user, for example by
adding \*QCGCrtlib.c\*U to the linkOption target parameter.
.PP
The following is a description of the currently implemented functions and
macros.  By convention, all function identifiers start with the letters
\*Q\fIFIX_\fP\*Q.  Some functions combine primitive operations in order to
minimize the need for temporary variables in nested arithmetic expressions.
.LP
.KS
 \-\-
.sp -1.0v
.Dh ARITHMETIC OPERATIONS
.Ds
compute `dst = op1 + op2'
.De
int FIX_Add ($ref(dst), $ref(op1),$ref(op2));
.KE
.Ds
compute `dst = op1 \- op2'
.De
int FIX_Sub ($ref(dst), $ref(op1),$ref(op2));
.Ds
compute `dst = op1 \(** op2'
.De
int FIX_Mul ($ref(dst), $ref(op1),$ref(op2));
.Ds
compute `dst = op1 / op2'
.De
int FIX_Div ($ref(dst), $ref(op1),$ref(op2));
.Ds
compute `dst += op1 \(** op2' 
.De
int FIX_MulAdd ($ref(dst), $ref(op1),$ref(op2));
.sp -0.3v
.Ds
dito, but weight the product by 2^weight before accumulating 
.De
int FIX_MulAddWeighted ($ref(dst), $ref(op1),$ref(op2), int weight);
.Ds
compute `dst = \-src' 
.De
int FIX_Complement ($ref(dst),$ref(src));
.Ds
shift right by n bits (or left if n \*(lo 0) 
.De
int FIX_ArithmeticShiftRight ($ref(dst), int n);
.\"
.Dh ASSIGNMENT 
.IP
int FIX_Assign ($ref(dst), $ref(src));
.br
int FIX_DoubleAssign ($ref(dst), double value);
.\"
.Dh CONVERSION 
.IP
double FIX_Fix2Double ($ref(src));
.br
int FIX_Fix2Int ($ref(src));
.\"
.Dh COMPARISION 
.Ds
return a value less than, equal to, or greater than 0, based upon
whether the value of op1 is less than, equal to or greater than
that of op2 
.De
int FIX_Compare ($ref(op1),$ref(op2));
.Dh MISCELLANEOUS 
.Ds
return the items of the fix triplet;  note that the precision
is trimmed according to the implementation limits 
.De
int FIX_GetLength ($ref(src));
.br
int FIX_GetIntBits ($ref(src));
.br
fix FIX_GetReference ($ref(src));
.Ds
macro to weight a fix triplet by 2^weight for function source operands
and 2^\-weight for function results
.De
FIX_Weight($ref(src_or_dst), int weight)
.Ds
set the entries of a fix_prec structure from a double value;  the intb field
will be just large enough to hold the integer part of the double, while the
len field will equal the maximum word length;  this function returns 1 if the
integer part would be larger than the total word length.
.De
int FIX_SetPrecisionFromDouble ($precision(var), double d);
.Ds
assign the maximum value that is representable in the given format
[or the minimum value if sign != 0] 
.De
void FIX_AssignMaxValue ($ref(dst), int sign);
.Ds
return TRUE if src represents the value 0.0 
.De
int FIX_IsZero ($ref(src));
.Ds
set the fix to 0.0 
.De
void FIX_SetToZero ($ref(dst));
.Ds
return the sign bit of a fix value
.De
int FIX_Sign($ref(src));
.LP
 \-\-
.PP
Most functions with integer return value - that is all arithmetic operations
as well as shifting and assignment -, return 1 if the operation caused an
overflow and 0 if not.  This return value is internally or'ed together with
the contents of the global variable \*Q\fIfix_overflow\fP\*Q, so that the
following scheme can be used to check sequences of contiguous fix operations
against overflow errors:
.Ps
       /* don't forget to reset the overflow flag */
       fix_overflow = 0;

       /* do the fix operations, for example: */
       FIX_Add(...);
       FIX_Mul(...);

       /* now check whether overflow occured */
       if (fix_overflow)
        ... do whatever you want, e.g. print an error message ...
.Pe
.LP
CGC stars should be derived from the CGCFix star that provides basic code
generation methods for overflow checking.
.\"
.Sh III. Precisions and PrecisionStates
.PP
As noted above, precisions hold the word length and the size of the integer
part for a fix value.  A precision can be derived from other precisions or can
be explicitly specified by the user.  In the latter case a container class is
required that parses the user supplied specification into the precision tuple.
.PP
Therefore a new state type named \*Q\fIPrecisionState\fP\*U is introduced -
\*Q\fIprecision\fP\*U in ptlang files -, designed to hold fixed-point precisions
in all kinds of domains, not only in the CGC domain.  The PrecisionState class
supports the computation of precisions from galaxy parameters of type integer
to allow the user to control the precision of all fixed-point values in a
universe by a small set of galaxy parameters.
.PP
The PrecisionState class parses strings of the form \*Q\fIn.m\fP\*U and
\*Q\fIn/m\fP\*U where \fIn\fP and \fIm\fP are integer expressions.  The first
format is derived from the implementation of the Fix class and contains the
Length and IntBits field of the precision explicitly, as in
\*Q\fI32-reserved.32-reserved-normalization\fP\*U (here \*Q\fIreserved\fP\*U
is a galaxy parameter defining the size of the reserved part of a word and
\*Q\fInormalization\fP\*U is the parameter defining the normalization factor
in bits).  The second format has been introduced to minimize the typing effort
for the user;  here \*Q\fIn/m\fP\*U is interpreted as \*Q\fIn\fP fraction bits
from a total word length of \fIm\fP bits\*Q.  Thus, the above example can be
more simply written as \*Q\fInormalization / 32-reserved\fP\*Q.  [Note that,
if the expression contains a dot, the \*Q\fI/\fP\*U is interpreted as the
division operator: \*Q\fI6/2.2\fP\*U is the same as \*Q\fI3.2\fP\*Q].
.PP
A precision state is very easily inserted into existing code:  it is
sufficient to change the declaration of the state holding a precision value
from \*Q\fIstring\fP\*U to \*Q\fIprecision\fP\*Q.  For compatibility reasons, the
PrecisionState class still recognizes references to string parameters in
the form \*Q\fI{name}\fP\*Q, though it is better to change the declaration
of the galaxy parameter as well and to omit the curly brackets \*Q\fI{}\fP\*Q.
.PP
The parser routines for precisions are also invoked from within FixStates and
FixArrayStates (in the latter case, precisions were not supported despite the
manual that said so).
.\"
.Sh IV. Extensions to the star library in the CGC domain
.PP
Most of the fixed-point stars from the SDF domain where ported to the CGC
domain with the same functionality,  except the SDFIIRFix star (there was no
CGCIIR star available that I could copy).
.PP
The stars are derived from the CGCFix star that provides support for overflow
checking.  As noted above, all CGC stars that need fix support should be
derived from this star.  This is to make sure that future extensions to the
fix support can be easily incorporated into the star's code.
.LP
.KS
New stars:
.IP
.ll -6n
.nh
CGCAddFix CGCCxToFix CGCFIRFix CGCFix CGCFixToCx
CGCFixToFix CGCFixToFloat CGCFloatToFix CGCGainFix CGCMpyFix
CGCRampFix CGCRectFix CGCSubFix
.hy
.ll
.KE
.\"
.Sh V. Known bugs and Todo list
.sp -1.2v
.IP \0 +2n
.RS
.IP \(bu +2.5n
It should be possible to specify alternate overflow handlers, not only the
\*Qsaturate\*U mode of class Fix.  This also applies to the rounding mode   
.IP \(bu +2.5n
It is not possible to derive the precision of an input port from an output
port of the same star (or vice versa), if the precision of the source
porthole is not set from inside the same setup routine.
.IP
Ideally it would suffice to set the precision of the source (or sink)
stars of a universe to determine the precision of each and every fixed-point
value in that universe.  The problem is, that the chronological order in
which a scheduler invokes the setup routines of the stars of a universe
cannot be predetermined, and, if it could, deriving the precision of an
input port from a predecessor star would need a different invocation order
than deriving a precision of an output port form a succeeding star.
.IP
A solution to this would be to let the user directly assign a precision to
a porthole in the form of a porthole parameter.  Then the precisions of the
portholes are already defined before the scheduler invokes the setup
routines.  This would require large kernel changes, however, so that the
benefits will certainly not remunerate the effort.

