defstar
{
    name { PCMread }
    domain { CGC }
    descriptor
    {
Read mu-law PCM from a file and convert to linear floating point.
The file can be the audio port /dev/audio, if supported by the workstation.
    }
    explanation
    {
This code is based on the description of the T1 system found in:
Simon Haykin, "Communication Systems," section 8.2 (Wiley 1983).
    }
    version { @(#)CGCPCMread.pl	1.10 1/1/96 }
    author { T. M. Parks }
    copyright {
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
    }
    location { CGC main library }

    state
    {
        name { fileName }
        type { string }
        default { "/dev/audio" }
        desc { Input file for PCM data.  If empty, use standard input. }
    }

    state
    {
        name { blockSize }
        type { int }
        default { 128 }
        desc { Number of samples to read. }
    }

    protected
    {
        int standardInput:1;
    }

    output
    {
	name { output }
	type { float }
    }

    codeblock (declarations)
    {
        int $starSymbol(file);
        unsigned char $starSymbol(buf)[$val(blockSize)];
    }

    codeblock (sharedDeclarations)
    {
	int $sharedSymbol(PCMread,offset)[8] =
	    { 0, 33, 99, 231, 495, 1023, 2079, 4191 };

	double $sharedSymbol(PCMread,convert)[256];
    }

    codeblock (sharedProcedure)
    {
	/* Convert from mu-law PCM to floating point [-1.0,1.0]. */
	double $sharedSymbol(PCMread,muLawToLin)(byte)
	unsigned char byte;
	{
	    double x;
	    int exponent, mantissa, magnitude;

	    byte = ~byte;
	    exponent = (byte >> 4) & 0x7;
	    mantissa = byte & 0xf;
	    magnitude = $sharedSymbol(PCMread,offset)[exponent] + (mantissa << (exponent+1));
	    x = magnitude / 8031.0;
	    if (byte & 0x80) x = -x;

	    return x;
	}
    }

    codeblock (sharedInit)
    {
	{
	    int i;
	    for (i = 0; i <= 0xff; i++)
		$sharedSymbol(PCMread,convert)[i] = $sharedSymbol(PCMread,muLawToLin)(i);
	}
    }

    codeblock (openFile)
    {
	/* Open file for reading. */
	if (($starSymbol(file) = open("$val(fileName)",O_RDONLY,0666)) == -1)
	{
		perror("$val(fileName)");
		exit(1);
	}
    }

    codeblock (noOpen)
    {
	/* Use standard input for reading. */
	$starSymbol(file) = 0;
    }

    codeblock (read)
    {
	/* Read data from file. */
	if (read($starSymbol(file), $starSymbol(buf), $val(blockSize)) != $val(blockSize))
	{
	    perror("$val(fileName)");
	    exit(1);
	}
    }

    codeblock (convert)
    {
	{
	    int i;
	    for (i=0; i<$val(blockSize); i++)
		$ref(output,i) = $sharedSymbol(PCMread,convert)[$starSymbol(buf)[$val(blockSize)-1-i]];
	}
    }

    codeblock (closeFile)
    {
        /* Close file. */
        if (close($starSymbol(file)) != 0)
        {
            perror("$val(fileName)");
            exit(1);
        }
    }

    setup
    {
        standardInput = (strcmp(fileName,"") == 0);
	output.setSDFParams(blockSize, blockSize-1);
    }

    initCode
    {
	addInclude("<math.h>");
	addDeclaration(declarations);
	if (addGlobal(sharedDeclarations,"$sharedSymbol(PCMread,PCMread)"))
	{
	    addProcedure(sharedProcedure);
	    addCode(sharedInit);
	}
	if (standardInput) addCode(noOpen);
	else
	{
	    addInclude("<fcntl.h>");
	    addCode(openFile);
	}
    }

    go
    {
	addCode(read);
	addCode(convert);
    }

    wrapup
    {
	if (!standardInput) addCode(closeFile);
    }

    exectime
    {
	return int(blockSize) * 14;
    }
}
