defstar
{
    name { PCMwrite }
    domain { CGC }
    descriptor
    {
Convert input to mu-law PCM and write it to a file.
The file can be the audio port /dev/audio, if supported by the workstation.
    }
    explanation
    {
This code is based on the description of the T1 system found in:
Simon Haykin, "Communication Systems," section 8.2 (Wiley 1983).
    }
    version { @(#)CGCPCMwrite.pl	1.10 1/1/96 }
    author { T. M. Parks }
    copyright {
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
    }
    location { CGC main library }

    state
    {
        name { fileName }
        type { string }
        default { "/dev/audio" }
        desc { Output file for PCM data.  If empty, use standard output. }
    }

    state
    {
        name { blockSize }
        type { int }
        default { 128 }
        desc { Number of samples to write. }
    }

    protected
    {
        int standardOutput:1;
    }

    input
    {
	name { input }
	type { float }
    }

    codeblock (declarations)
    {
        int $starSymbol(file);
        unsigned char $starSymbol(buf)[$val(blockSize)];
    }

    codeblock (sharedDeclarations)
    {
	int $sharedSymbol(PCMwrite,threshold)[8] =
		    { 0, 32, 96, 224, 480, 992, 2016, 4064 };

	int $sharedSymbol(PCMwrite,exponent)[256] =
	    { 0,1,1,2,2,2,2,3,3,3,3,3,3,3,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
	    5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
	    6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
	    6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
	    7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
	    7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
	    7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
	    7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7};
    }

    codeblock (openFile)
    {
	/* Open file for writing. */
	if (($starSymbol(file) = open("$val(fileName)",O_WRONLY|O_CREAT,0666)) == -1)
	{
		perror("$val(fileName)");
		exit(1);
	}
    }

    codeblock (noOpen)
    {
	/* Use standard output for writing. */
	$starSymbol(file) = 1;
    }

    codeblock (convert)
    {
	/* Convert from floating point [-1.0,1.0] to mu-law PCM. */
	{
	    int i;
	    for(i=0; i<$val(blockSize); i++)
	    {
		int exponent, mantissa, magnitude;
		unsigned char byte;

		magnitude = fabs($ref(input,i)) * 8031.0;
		if (magnitude > 8031) magnitude = 8031;
		exponent = $sharedSymbol(PCMwrite,exponent)[magnitude >> 5];
		mantissa = (magnitude-$sharedSymbol(PCMwrite,threshold)[exponent]) >> (exponent+1);
		byte = (exponent << 4) | mantissa;
		if ($ref(input,i) < 0.0) byte |= 0x80;
		$starSymbol(buf)[$val(blockSize)-1-i] = ~byte;
	    }
	}
    }

    codeblock (write)
    {
	/* Write data to file. */
	if (write($starSymbol(file), $starSymbol(buf), $val(blockSize)) != $val(blockSize))
	{
	    perror("$val(fileName)");
	    exit(1);
	}
    }

    codeblock (closeFile)
    {
        /* Close file. */
        if (close($starSymbol(file)) != 0)
        {
            perror("$val(fileName)");
            exit(1);
        }
    }

    setup
    {
        standardOutput = (strcmp(fileName,"") == 0);
	input.setSDFParams(blockSize, blockSize-1);
    }

    initCode
    {
	addInclude("<math.h>");
	addDeclaration(declarations);
	addGlobal(sharedDeclarations,"$sharedSymbol(PCMwrite,PCMwrite)");
	if (standardOutput) addCode(noOpen);
	else
	{
	    addInclude("<fcntl.h>");
	    addCode(openFile);
	}
    }

    go
    {
	addCode(convert);
	addCode(write);
    }

    wrapup
    {
	if (!standardOutput) addCode(closeFile);
    }

    exectime
    {
	return int(blockSize) * 19;
    }
}
