defstar
{
    name { SunSound }
    derivedFrom { PCMwrite }
    domain { CGC }
    desc {
Convert input to mu-law PCM and write to the Sun audio port /dev/audio.
The input is expected to be in the range from -1.0 to 1.0.
If the "aheadLimit" parameter is non-negative, then it specifies
the maximum number of samples that the program will be allowed to compute
ahead of real-time.
    }
    version { @(#)CGCSunSound.pl	1.14 3/8/96 }
    author { E. A. Lee }
    copyright {
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
    }

    defstate {
        name {volume}
        type{int}
        default {"25"}
        desc {The volume of the audio output (percent of full volume).}
	attributes { A_GLOBAL }
    }

    defstate {
        name {aheadLimit}
        type{int}
        default {"-1"}
        desc {If non-negative, the number of samples ahead that can computed.}
    }

    codeblock (declarations) {
        unsigned $starSymbol(count);
    }

    codeblock (globals) {
	int $starSymbol(ctlfile);
    }

    codeblock (openFile) {
	/* Open control device. */
	if (($starSymbol(ctlfile) = open("/dev/audioctl",O_RDWR,0777)) == -1) {
		perror("/dev/audioctl");
		exit(1);
	}
        $starSymbol(count) = 0;
    }

    codeblock (closeFile) {
	/* Close control device. */
	if (close($starSymbol(ctlfile)) != 0) {
	    perror("/dev/audioctl");
	    exit(1);
	}
    }

    codeblock (sync) {
        {
        audio_info_t info;
	/* Wait for some samples to drain */
	do {
	    /* the cast below is to prevent warnings */
            ioctl($starSymbol(file), AUDIO_GETINFO, (caddr_t)(&info));
        } while ((int) ($starSymbol(count) - info.play.samples) > $val(aheadLimit));
        $starSymbol(count) += $val(blockSize);
        }
    }

    codeblock (setvDef) {
        /* The following function should really be shared among */
        /* all instances of the star.  Moreover, the star should */
        /* request notification of changes in the I/O status, and */
        /* should adjust its slider accordingly. */
        static void $starSymbol(setv) () {
            audio_info_t info;

            AUDIO_INITINFO(&info);
            info.play.gain = AUDIO_MAX_GAIN*$ref(volume)/100;
            ioctl($starSymbol(ctlfile), AUDIO_SETINFO, (caddr_t)(&info));
        }
    }

    setup {
	fileName.clearAttributes(A_SETTABLE);
	CGCPCMwrite::setup();
    }
    // For the following hack
    ccinclude { "compat.h" }
    initCode {
	CGCPCMwrite::initCode();

	addInclude("<unistd.h>");
	addInclude("<sys/ioctl.h>");
// FIXME: Hack alert: The following is not entirely satisfactory,
// since the generated code may not be compiled on the same platform
// that this star is compiled on, and Sun, unfortunately, decided to
// put this file in two different places on their two O/S's. 
#if defined(PTSUN4)  && !defined(PTSOL2_CFRONT)
	addInclude("<sun/audioio.h>");
#else
	addInclude("<sys/audioio.h>");
#endif
	addDeclaration(declarations);
        addGlobal(globals);

	addCode(openFile);
        addProcedure(setvDef);

        // update state volume
        addCode("$starSymbol(setv) ();\n");
    }

    go {
	addCode(convert);
	if ((int)aheadLimit >= 0 ) addCode(sync);
	addCode(write);
    }

    wrapup {
	CGCPCMwrite::wrapup();
	addCode(closeFile);
    }

    exectime {
	return 100;  /* unreliable data */
    }
}
