static const char file_id[] = "SDFFileEval.pl";
// .cc file generated from SDFFileEval.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFFileEval.h"
#include "FileMessage.h"
#include <iostream.h>

const char *star_nm_SDFFileEval = "SDFFileEval";

const char* SDFFileEval :: className() const {return star_nm_SDFFileEval;}

ISA_FUNC(SDFFileEval,SDFTclScript);

Block* SDFFileEval :: makeNew() const { LOG_NEW; return new SDFFileEval;}

# line 172 "SDFFileEval.pl"
extern "C" {
	    #include "ptk.h"
	}

SDFFileEval::SDFFileEval ()
{
	setDescriptor("Evaluate the \"command\" parameter in Tcl. Inputs are referenced in\nthe comand as \"input#1\", \"input#2\", etc.  Outputs are file names,\nand are referenced in the command as \"output#1\", \"output#2\", etc.\nFor example, if \"command\" is \"exec sed -e /foo/s//bar/ input#1 > output#1\",\nthen the program \"sed\" will be used to replace all instances of \"foo\"\nin the input file given by the first input to yield the output file\non the first output.");
	addPort(out.setPort("out",this,FILEMSG));
	addState(command.setState("command",this,"exec cat input#1 > output#1","The command line to execute."));
	addState(conditional.setState("conditional",this,"NO","If YES, evaluate only if something has changed."));
	addState(output_filenames.setState("output_filenames",this,"","If no filenames are given, temporary files will be used."));
	addState(save_output_files.setState("save_output_files",this,"NO","If temporary files are used, this specifies whether they persist."));
	addState(show_evaluation.setState("show_evaluation",this,"YES","Show evaluation by highlighting the block in green"));

# line 168 "SDFFileEval.pl"
tcl_file.clearAttributes(A_SETTABLE);
	output.setAttributes(P_HIDDEN);
}

void SDFFileEval::begin() {
# line 157 "SDFFileEval.pl"
tcl_file = "$PTOLEMY/src/domains/sdf/dfm/stars/fileeval.tcl";

	if (sourceIfNeeded("fileeval_evalneeded",
	        "$PTOLEMY/src/domains/sdf/dfm/stars/fileevalsupport.tcl")
		== 0) {
	    return;
	}

        SDFTclScript::begin();
}

void SDFFileEval::setup() {
# line 145 "SDFFileEval.pl"
if (output_filenames.size() == 0) {
	    useTmpFiles = 1;
	} else {
	    useTmpFiles = 0;
	    if (output_filenames.size() != out.numberPorts()) {
		Error::abortRun(*this,
		"Number of output filenames does not match "
		"the number of outputs");
	    }
	}
}

void SDFFileEval::go() {
# line 257 "SDFFileEval.pl"
if (evalneeded()) {
	    if (int(show_evaluation)) {
		InfString cmd = "ptkHighlight ";
		cmd << fullName();
		// The following specifies a color in the arcane way 
		// required by vem: RGB. NOTE: Tycho incompatibility.
		cmd << " 0 65535 0";
		Tcl_GlobalEval(ptkInterp, (char*)cmd);
	    }
	    SDFTclScript::go();
	    if (int(show_evaluation)) {
		InfString cmd = "ptkClearHighlights";
		Tcl_GlobalEval(ptkInterp, (char*)cmd);
	    }	    
	}
}


int SDFFileEval::evalneeded ()
{
# line 199 "SDFFileEval.pl"
// Construct a list of output filenames for Tcl.
	    InfString outnames;
	    MPHIter nextp(out);
	    for (int i = 0; i < out.numberPorts(); i++) {
		FileMessage* fm;
		if (useTmpFiles) {
		    fm = new FileMessage();
		} else {
		    fm = new FileMessage((char*)output_filenames[i]);
		}
		if (int(save_output_files)) {
		    fm->setTransient(0);
		}
		outnames += fm->fileName();
		Envelope pkt(*fm);
		
		// Now send the filename to the output
		PortHole* p = nextp++;
		(*p)%0 << pkt;
	    }
	    char* buf = (char*)outnames;
	    if (!buf) {buf = ""; }
	    Tcl_SetVar2(ptkInterp, tcl.id(), "outputfilenames",
	    buf, TCL_GLOBAL_ONLY);

	    InfString cmd = "[fileeval_evalneeded ";
	    cmd << tcl.id() << "]";
	    int result;
	    if (Tcl_ExprBoolean(ptkInterp, (char*)cmd, &result) != TCL_OK) {
		Error::abortRun(*this, ptkInterp->result);
		return 0;
	    } else {
		return result;
	    }
}


int SDFFileEval::sourceIfNeeded (const char* proc, const char* filename)
{
# line 245 "SDFFileEval.pl"
InfString cmd;
	    cmd << "if {[info commands " << proc << "] == {}} {\n";
	    cmd << "    source " << filename << "\n";
	    cmd << "}";
	    if (Tcl_GlobalEval(ptkInterp, (char*)cmd) != TCL_OK) {
		Error::abortRun(*this, ptkInterp->result);
		return 0;
	    }
	    return 1;
}


// prototype instance for known block list
static SDFFileEval proto;
static RegisterBlock registerBlock(proto,"FileEval");
