static const char file_id[] = "SDFMatlab_M.pl";
// .cc file generated from SDFMatlab_M.pl by ptlang
/*
Copyright (c) 1990-1996 The Regents of the University of California.
All rights reserved.
See the file $PTOLEMY/copyright for copyright notice,
limitation of liability, and disclaimer of warranty provisions.
 */

#ifdef __GNUG__
#pragma implementation
#endif

#include "SDFMatlab_M.h"

const char *star_nm_SDFMatlab_M = "SDFMatlab_M";

const char* SDFMatlab_M :: className() const {return star_nm_SDFMatlab_M;}

ISA_FUNC(SDFMatlab_M,SDFMatlab);

Block* SDFMatlab_M :: makeNew() const { LOG_NEW; return new SDFMatlab_M;}

SDFMatlab_M::SDFMatlab_M ()
{
	setDescriptor("Evaluate a Matlab function if inputs are given or\nevaluate a Matlab command if no inputs are given.");
	addPort(input.setPort("input",this,ANYTYPE));
	addPort(output.setPort("output",this,FLOAT_MATRIX_ENV));
	addState(MatlabSetUp.setState("MatlabSetUp",this,"","The Matlab command to execute during the begin method of the Matlab star.\nDuring the begin procedure, there is no data passing into or out of the star."));
	addState(MatlabFunction.setState("MatlabFunction",this,"","The Matlab command to execute each time the star fires.\nThe values of the input and output ports may be accessed using the\nPtolemy notation input#i and output#i, where i is the port number,\nas explained above."));
	addState(MatlabWrapUp.setState("MatlabWrapUp",this,"","The Matlab command to execute during the wrapup procedure of the Matlab star.\nDuring the wrapup procedure, there is no data passing into or out of the star."));

# line 101 "SDFMatlab_M.pl"
matlabInputMatrices = 0;
		matlabOutputMatrices = 0;

		matlabInputNames = 0;
		matlabOutputNames = 0;

		numInputs = 0;
		numOutputs = 0;
}

void SDFMatlab_M::wrapup() {
# line 226 "SDFMatlab_M.pl"
const char* wrapupCommand = (const char*) MatlabWrapUp;
		if ( wrapupCommand && *wrapupCommand ) {
		  InfString matlabCommand = wrapupCommand;
		  int err = matlabInterface->EvaluateUserCommand(matlabCommand);
		  if ( err ) {
		    Error::abortRun( *this, matlabInterface->GetErrorString() );
		  }
		}
}

void SDFMatlab_M::begin() {
# line 133 "SDFMatlab_M.pl"
// run the begin method of the base star (start Matlab, etc.)
		SDFMatlab::begin();

		// free any existing memory
		matlabInterface->FreeMatlabMatrices(matlabInputMatrices,
						    numInputs);
		delete [] matlabInputMatrices;
		matlabInputMatrices = 0;
		matlabInterface->FreeMatlabMatrices(matlabOutputMatrices,
						    numOutputs);
		delete [] matlabOutputMatrices;
		matlabOutputMatrices = 0;

		matlabInterface->FreeStringArray(matlabInputNames, numInputs);
		delete [] matlabInputNames;
		matlabInputNames = 0;
		matlabInterface->FreeStringArray(matlabOutputNames, numOutputs);
		delete [] matlabOutputNames;
		matlabOutputNames = 0;

		// allocate Matlab input matrices and generate their names
		if ( numInputs > 0 ) {
		  matlabInputMatrices = new MatrixPtr[numInputs];
		  for ( int k = 0; k < numInputs; k++ ) {
		    matlabInputMatrices[k] = 0;
		  }
		  matlabInputNames = new char*[numInputs];
		  matlabInterface->NameMatlabMatrices(matlabInputNames,
		  				      numInputs, "input");
		}

		// allocate Matlab output matrices and generate their names
		if ( numOutputs > 0 ) {
		  matlabOutputMatrices = new MatrixPtr[numOutputs];
		  for ( int k = 0; k < numOutputs; k++ ) {
		    matlabOutputMatrices[k] = 0;
		  }
		  matlabOutputNames = new char*[numOutputs];
		  matlabInterface->NameMatlabMatrices(matlabOutputNames,
		  				      numOutputs, "output");
		}

		// create the command to be sent to the Matlab interpreter
		matlabInterface->BuildMatlabCommand(
			matlabInputNames, numInputs,
			(const char *) MatlabFunction,
			matlabOutputNames, numOutputs);

		// evaluate the MatlabSetUp if provided
		const char* setupCommand = (const char*) MatlabSetUp;
		if ( setupCommand && *setupCommand ) {
		  InfString matlabCommand = setupCommand;
		  int err = matlabInterface->EvaluateUserCommand(matlabCommand);
		  if ( err ) {
		    Error::abortRun( *this, matlabInterface->GetErrorString() );
		  }
		}
}

SDFMatlab_M::~SDFMatlab_M() {
# line 237 "SDFMatlab_M.pl"
matlabInterface->FreeMatlabMatrices(matlabInputMatrices,
						   numInputs);
		delete [] matlabInputMatrices;
		matlabInterface->FreeMatlabMatrices(matlabOutputMatrices,
						   numOutputs);
		delete [] matlabOutputMatrices;

		matlabInterface->FreeStringArray(matlabInputNames, numInputs);
		delete [] matlabInputNames;
		matlabInterface->FreeStringArray(matlabOutputNames, numOutputs);
		delete [] matlabOutputNames;
}

void SDFMatlab_M::setup() {
# line 112 "SDFMatlab_M.pl"
// run the setup method of the base star
		SDFMatlab::setup();

		// set the number of input and output ports
		numInputs = input.numberPorts();
		numOutputs = output.numberPorts();

		// check data type of output ports
		if ( numOutputs > 0 ) {
		  DataType outType = output.type();
		  if ( outType != FLOAT_MATRIX_ENV &&
		       outType != COMPLEX_MATRIX_ENV ) {
		    Error::abortRun(*this,
				    "The output ports may only support float",
				    "matrix or complex matrix envelopes, not",
				    output.type());
		  }
		}
}

void SDFMatlab_M::go() {
# line 193 "SDFMatlab_M.pl"
// convert Ptolemy input matrices to Matlab matrices
		processInputMatrices();

		// evaluate the Matlab command (non-zero means error)
		StringList errmsg;
		int merror = matlabInterface->EvaluateUserCommand();
		if ( merror ) {
		  errmsg = matlabInterface->GetErrorString();
		  Error::abortRun( *this, matlabInterface->GetErrorString() );
		}
		else {
		  // convert Matlab matrices to Ptolemy matrices
		  merror = processOutputMatrices();
		  errmsg = "Could not convert the Matlab output matrices to Ptolemy matrices";
		}

		// Free dynamic memory
		matlabInterface->FreeMatlabMatrices(matlabInputMatrices,
						    numInputs);
		matlabInterface->UnsetMatlabVariable(matlabInputNames,
						     numInputs);
		matlabInterface->FreeMatlabMatrices(matlabOutputMatrices,
						    numOutputs);
		matlabInterface->UnsetMatlabVariable(matlabOutputNames,
						     numOutputs);

		// Report any errors that occurred
		if ( merror ) {
		  Error::abortRun( *this, errmsg );
		}
}


int SDFMatlab_M::processInputMatrices ()
{
# line 257 "SDFMatlab_M.pl"
int errorFlag = FALSE;

		// allocate memory for Matlab matrices
		MPHIter nexti(input);
		for ( int i = 0; i < numInputs; i++ ) {
		  // read a reference to the matrix on input port i
		  int errflag = FALSE;
		  PortHole *iportp = nexti++;
		  DataType portType = iportp->resolvedType();
		  Matrix *matlabMatrix =
		  	matlabInterface->PtolemyToMatlab(
				(*iportp)%0, portType, &errflag);

		  // check for error in the call to PtolemyToMatlab
		  if ( errflag ) {
		    errorFlag = TRUE;
		    StringList errstr;
		    errstr = "Unsupported data type ";
		    errstr << portType << " on input port " << i+1 << ".";
		    Error::warn(*this, errstr);
		  }

		  // Give the current matrix a name
		  matlabInterface->NameMatlabMatrix(matlabMatrix,
		  				   matlabInputNames[i]);

		  // let Matlab know about the new Matlab matrix we've defined
		  // FIXME: Memory Leak
		  matlabInterface->MatlabEnginePutMatrix(matlabMatrix);

		  // save the pointer to the new Matlab matrix for deallocation
		  matlabInputMatrices[i] = matlabMatrix;
		}

		return( errorFlag );
}


int SDFMatlab_M::processOutputMatrices ()
{
# line 302 "SDFMatlab_M.pl"
// possible error messages and flags
		int incompatibleOutportPort = FALSE;
		int matlabFatalError = FALSE;
		int nullMatlabMatrix = FALSE;
		StringList merrstr, mverbstr, nerrstr, nverbstr;

		// iterate through the output ports
		MPHIter nextp(output);
		for ( int j = 0; j < numOutputs; j++ ) {
		  // current output porthole & advance nextp to next porthole
		  PortHole *oportp = nextp++;
		  DataType portType = oportp->resolvedType();

		  // create a new Matlab matrix for deallocation and save ref.
		  // FIXME: Memory leak
		  Matrix *matlabMatrix =
		  matlabInterface->MatlabEngineGetMatrix(matlabOutputNames[j]);

		  // save the pointer to the new Matlab matrix for deallocation
		  matlabOutputMatrices[j] = matlabMatrix;

		  // check to make that the Matlab matrix is defined
		  if ( matlabMatrix == 0 ) {
		    if ( ! nullMatlabMatrix ) {
		      nullMatlabMatrix = TRUE;
		      nerrstr = "For the Matlab command ";
		      nerrstr << matlabInterface->GetMatlabCommand() << ", ";
		      nverbstr = " is not defined.";
		    }
		    else {
		      nerrstr << " and ";
		      nverbstr = " are not defined.";
		    }
		    nerrstr << matlabOutputNames[j];
		    continue;
		  }

		  StringList matrixId = matlabOutputNames[j];
		  matrixId << " on output port " << j;
		  int errflag = FALSE;
		  int warnflag = FALSE;
		  int badport = matlabInterface->MatlabToPtolemy(
		  			(*oportp)%0, portType, matlabMatrix,
					&warnflag, &errflag);
		  incompatibleOutportPort = incompatibleOutportPort || badport;

		  if ( warnflag ) {
		    Error::warn(*this, matlabInterface->GetWarningString() );
		  }

		  if ( errflag ) {
		    if ( ! matlabFatalError ) {
		      matlabFatalError = TRUE;
		      merrstr = "For the Matlab command ";
		      merrstr << matlabInterface->GetMatlabCommand() << ", ";
		      mverbstr = " is not a full matrix.";
		    }
		    else {
		      merrstr << " and ";
		      mverbstr = " are not full matrices.";
		    }
		    merrstr << matlabOutputNames[j];
		  }
		}

		// print warning messages and return error status
		if ( incompatibleOutportPort ) {
		  Error::warn(*this,
			"Output port data type is neither a floating-point",
			"matrix nor a complex-valued matrix: the setup() ",
			"method should have flagged this error.");
		}
		if ( nullMatlabMatrix ) {
		  Error::warn(*this, nerrstr, nverbstr);
		}
		if ( matlabFatalError ) {
		  Error::warn(*this, merrstr, mverbstr );
		}

		return( incompatibleOutportPort ||
			nullMatlabMatrix ||
			matlabFatalError );
}


// prototype instance for known block list
static SDFMatlab_M proto;
static RegisterBlock registerBlock(proto,"Matlab_M");
