# Demo suite for widgets in tycho.
#
# Author: Edward A. Lee
#
# Unlike the test suite, this script is highly interactive.
# It walks you through most of the features of the tycho classes.
#
# Limitations:
#  - This is assumed to be run in $PTOLEMY/tycho/kernel_itk.
#  - Everything comes up in the center.  Use .tyc file later.
#

namespace tycho
import add tycho

# FIXME: These should be auto-loaded.
source Color.tcl
source TopLevel.itcl
source Dismiss.itcl
source Font.itcl
source Options.tcl
source DialogWindow.itcl
source Message.itcl
source ErrorMessage.itcl
source YesNoQuery.itcl
source YesNoCancel.itcl


############################################################################
#### removeobj
# This procedure removes an object if it exists.
proc tycho::removeobj {name} {
    if {[info object $name] != ""} {
	delete object $name
    }
}

############################################################################
#### show
# This procedure displays the description followed by the command
# in a dialog window.  Clicking on the Yes button then causes the
# command to execute.  This is done with a modal dialog in order to
# pace the execution.
#
proc tycho::show {desc args} {
    set command [join $args "\n    "]
    if [askuser "$desc\nThe commands are:\n\n   \
	    $command\n\nDo you want to execute them?"] {
	eval $command
	inform \
		"The command you just executed was:\n\n   \
		$command\n\nClick to continue"
    }
}

########################################################################
#### start
# Open a master console window with information.

########################################################################
# Demo TopLevel
show "Create a top-level window with a specified geometry." \
	"TopLevel .t -geometry 300x300+0+0" \
	"wm deiconify .t"

if [winfo exists .t] {
    show "Center the window on the screen." \
	    ".t centerOnScreen"
    delete object .t
}

########################################################################
# Demo various messages
show "There are several modal mechanisms for posting messages. \
	Modal means that execution is blocked until the message \
	is dismissed." \
	"inform {This message is created with the inform command}" \
	"warn {This message is created with the warn command}" \
	"tkerror {This message is created with the tkerror command}"

show "There are also some non-modal mechanisms for posting messages. \
	Non-modal means that execution continues as soon as the \
	message is posted." \
	"post {This message is created with the post command}"

show "You can customize the dialog boxes by adding buttons, for example." \
	"Message .m -text {Adding buttons}" \
	"wm deiconify .m" \
	".m addButton pushme -text {Push Me} -command {post thanks}"

show "You can ask the user questions, and act occordingly" \
	{if [askuser "Are you smart?"] {post "You said yes!"}}

show "You can customize the action taken in response to buttons." \
	{YesNoQuery .y -text "Are you smart" -yescommand {post "You said yes!"}} \
	{.y centerOnScreen}

show "A dialog with three buttons can also be created." \
	"YesNoCancel::newModal -text {Choose one} \\" \
	"    -yescommand {puts yes} \\" \
	"    -nocommand {puts no} \\" \
	"    -cancelcommand {puts cancel}"

########################################################################
# Demo Dismiss windows
show "A window with a dismiss button." \
	"Dismiss .d" \
	"wm deiconify .d"
